/* 
 * Created on 2-jul-2006 by ronald.blankendaal
 * 
 * @file $RCSfile: EditProfileDialog.java,v $
 * @version $Revision: 1.29 $ 
 * @author $Author: ronald $ (last checked in by) 
 * @date $Date: 2006/08/30 16:56:46 $ (UTC date of last check in)
 */

package com.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Dialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import com.db.Database;
import com.model.Autoexec;
import com.model.Configuration;
import com.model.Developer;
import com.model.DosboxVersion;
import com.model.Genre;
import com.model.Mount;
import com.model.Profile;
import com.model.Publisher;
import com.model.Status;
import com.model.Template;
import com.model.Year;
import com.util.ListUtils;
import swing2swt.layout.BorderLayout;


public class EditProfileDialog extends Dialog {
  
  private TabItem mountingTabItem;
  private TabItem infoTabItem;
  private Text title;
  private Combo developer;
  private Combo publisher;
  private Combo genre;
  private Combo year;
  private Combo status;
  private Text link1;
  private Text link2;
  private Text notes;
  private Button reloadSettingsButton;
  private Combo dbversion;
  private Combo template;
  private Combo priority_inactive;
  private Combo priority_active;
  private Button waitonerror;
  private Button exit;
  private Button fullscreen;
  private Button fulldouble;
  private Button aspect;
  private Text language;
  private Text captures;
  private Combo machine;
  private Combo core;
  private Combo cycles;
  private Combo cycles_up;
  private Combo cycles_down;
  private Button timesynched;
  private Combo memsize;
  private Button xms;
  private Button ems;
  private Combo umb;
  private Button loadfix;
  private Combo loadfix_value;
  private Combo output;
  private Combo frameskip;
  private Combo scaler;
  private Combo fullresolution;
  private Combo windowresolution;
  private Combo tandy;
  private Combo tandyrate;
  private Text midi_config;
  private Combo device;
  private Combo mpu401;
  private Text ultradir;
  private Combo gusrate;
  private Combo irq2;
  private Combo dma2;
  private Combo dma1;
  private Combo irq1;
  private Combo gusbase;
  private Combo pcrate;
  private Combo hdma;
  private Combo dma;
  private Combo irq;
  private Combo sbbase;
  private Combo oplrate;
  private Combo oplmode;
  private Combo sbtype;
  private Combo prebuffer;
  private Combo blocksize;
  private Combo rate;
  private Button nosound;
  private Button mixer;
  private Button pcspeaker;
  private Button disney;
  private Button gus;
  private Button dosButton;
  private Button booterButton;
  private Button usescancodes;
  private Button autolock;
  private Combo sensitivity;
  private Text serial1;
  private Text serial2;
  private Text serial3;
  private Text serial4;
  private Combo joysticktype;
  private Text mapperfile;
  private Button ipx;
  private List mountingpoints;
  private Text main;
  private Text main_params;
  private Text setup;
  private Text setup_params;
  private Text img1;
  private Text img2;
  
  protected Shell shell;
  private Database db;
  private java.util.List<Developer> developersList;
  private java.util.List<Publisher> publishersList;
  private java.util.List<Genre> genresList;
  private java.util.List<Year> yearsList;
  private java.util.List<DosboxVersion> dbversionsList;
  private java.util.List<Template> templatesList;
  private java.util.List<Status> statusList;
  private Configuration profileConf; 
  protected Object result;

  /**
   * Create the dialog
   * @param parent
   * @param style
   */
  public EditProfileDialog(Shell parent, int style) {
    super(parent, style);
  }

  /**
   * Create the dialog
   * @param parent
   */
  public EditProfileDialog(Shell parent) {
    this(parent, SWT.NONE);
  }
  
  public void setDatabase(Database db) {
    this.db = db;
  }
  
  public void setProfile(Profile prof) {
    this.result = prof;
  }

  /**
   * Open the dialog
   * @return the result
   */
  public Object open() {
    createContents();
    shell.open();
    shell.layout();
    Display display = getParent().getDisplay();
    while (!shell.isDisposed()) {
      if (!display.readAndDispatch())
        display.sleep();
    }
    return result;
  }

  /**
   * Create contents of the dialog
   */
  protected void createContents() {
    developersList = db.readDevelopersList();
    publishersList = db.readPublishersList();
    genresList = db.readGenresList();
    yearsList = db.readYearsList();
    dbversionsList = db.readDosboxVersionsList();
    templatesList = db.readTemplatesList();
    statusList = db.readStatusList();
    
    shell = new Shell(getParent(), SWT.TITLE | SWT.CLOSE | SWT.BORDER | SWT.RESIZE | SWT.APPLICATION_MODAL);
    shell.setLayout(new BorderLayout(0, 0));
    shell.setSize(610, 475);
    
    if (result != null) {
      // meaning we are essentially editing an existing profile
      shell.setText("Edit Profile " + ((Profile)result).getTitle() + " (" + ((Profile)result).getId() + ")");
    } else {
      shell.setText("Add Profile");
    }

    final TabFolder tabFolder = new TabFolder(shell, SWT.NONE);

    infoTabItem = new TabItem(tabFolder, SWT.NONE);
    infoTabItem.setText("Info");

    final Composite composite = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout = new GridLayout();
    gridLayout.numColumns = 4;
    composite.setLayout(gridLayout);
    infoTabItem.setControl(composite);

    final Label titleLabel = new Label(composite, SWT.NONE);
    titleLabel.setText("Title");

    title = new Text(composite, SWT.BORDER);
    title.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 3, 1));

    final Label developerLabel = new Label(composite, SWT.NONE);
    developerLabel.setText("Developer");

    developer = new Combo(composite, SWT.NONE);
    developer.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    developer.setVisibleItemCount(15);

    final Label publisherLabel = new Label(composite, SWT.NONE);
    publisherLabel.setLayoutData(new GridData());
    publisherLabel.setText("Publisher");

    publisher = new Combo(composite, SWT.NONE);
    publisher.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    publisher.setVisibleItemCount(15);
    
    final Label genreLabel = new Label(composite, SWT.NONE);
    genreLabel.setText("Genre");

    genre = new Combo(composite, SWT.NONE);
    genre.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    genre.setVisibleItemCount(15);

    final Label yearLabel = new Label(composite, SWT.NONE);
    yearLabel.setLayoutData(new GridData());
    yearLabel.setText("Year");

    year = new Combo(composite, SWT.NONE);
    year.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    year.setVisibleItemCount(15);
    
    final TabItem generalTabItem = new TabItem(tabFolder, SWT.NONE);
    generalTabItem.setText("General");

    final Composite composite_1 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_1 = new GridLayout();
    gridLayout_1.numColumns = 3;
    composite_1.setLayout(gridLayout_1);
    generalTabItem.setControl(composite_1);

    final Label dosboxVersionLabel = new Label(composite_1, SWT.NONE);
    dosboxVersionLabel.setText("Dosbox version");

    dbversion = new Combo(composite_1, SWT.READ_ONLY);
    final GridData gridData_7 = new GridData(SWT.FILL, SWT.CENTER, true, false);
    dbversion.setLayoutData(gridData_7);

    reloadSettingsButton = new Button(composite_1, SWT.NONE);
    reloadSettingsButton.setText("Reload Settings");
    reloadSettingsButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        DosboxVersion selectedDBVersion = dbversionsList.get(dbversion.getSelectionIndex());
        profileConf = new Configuration(selectedDBVersion.getPath() + MainWindow.DOSBOX_CONF);
        selectSettingsByConfiguration(profileConf);
      }
    });

    final Label templateLabel = new Label(composite_1, SWT.NONE);
    templateLabel.setText("Template");

    template = new Combo(composite_1, SWT.READ_ONLY);
    final GridData gridData_6 = new GridData(SWT.FILL, SWT.CENTER, true, false);
    template.setLayoutData(gridData_6);

    final Button reloadSettingsButton_1 = new Button(composite_1, SWT.NONE);
    reloadSettingsButton_1.setText("Reload Settings");
    reloadSettingsButton_1.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
      	if (template.getSelectionIndex() != -1) {
	        DosboxVersion selectedDBVersion = dbversionsList.get(dbversion.getSelectionIndex());
	        profileConf = new Configuration(selectedDBVersion.getPath() + MainWindow.DOSBOX_CONF);
	        Template templ = templatesList.get(template.getSelectionIndex());
	        profileConf.parseConfigurationFile(MainWindow.TEMPLATES_DIR + System.getProperty("file.separator") + templ.getId() + ".conf");
	        selectSettingsByConfiguration(profileConf);
      	}
      }
    });

    final Label label = new Label(composite_1, SWT.SEPARATOR | SWT.HORIZONTAL);
    label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 3, 1));
    new Label(composite_1, SWT.NONE);

    final Label activeLabel = new Label(composite_1, SWT.NONE);
    activeLabel.setText("Active");

    final Label inactiveLabel = new Label(composite_1, SWT.NONE);
    inactiveLabel.setText("Inactive");

    final Label priorityLabel = new Label(composite_1, SWT.NONE);
    priorityLabel.setText("Priority");

    priority_active = new Combo(composite_1, SWT.READ_ONLY);
    priority_active.setItems(new String[] {"lowest", "lower", "normal", "higher", "highest", "pause"});
    final GridData gridData_5 = new GridData(SWT.LEFT, SWT.CENTER, true, false);
    gridData_5.minimumWidth = 150;
    priority_active.setLayoutData(gridData_5);

    priority_inactive = new Combo(composite_1, SWT.READ_ONLY);
    priority_inactive.setItems(new String[] {"lowest", "lower", "normal", "higher", "highest", "pause"});
    priority_inactive.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));

    final Label waitOnErrorLabel = new Label(composite_1, SWT.NONE);
    waitOnErrorLabel.setText("Wait On Error");

    waitonerror = new Button(composite_1, SWT.CHECK);
    new Label(composite_1, SWT.NONE);

    final Label exitAfterwardsLabel = new Label(composite_1, SWT.NONE);
    exitAfterwardsLabel.setText("Exit afterwards");

    exit = new Button(composite_1, SWT.CHECK);
    new Label(composite_1, SWT.NONE);

    final Label languageFileLabel = new Label(composite_1, SWT.NONE);
    languageFileLabel.setLayoutData(new GridData());
    languageFileLabel.setText("Language File");

    language = new Text(composite_1, SWT.BORDER);
    final GridData gridData_4 = new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1);
    language.setLayoutData(gridData_4);
    
    final TabItem displayTabItem = new TabItem(tabFolder, SWT.NONE);
    displayTabItem.setText("Display");

    final Composite composite_2 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_5 = new GridLayout();
    gridLayout_5.numColumns = 4;
    composite_2.setLayout(gridLayout_5);
    displayTabItem.setControl(composite_2);

    final Label outputLabel = new Label(composite_2, SWT.NONE);
    outputLabel.setText("Output");

    output = new Combo(composite_2, SWT.READ_ONLY);
    output.setItems(new String[] {"ddraw", "direct3d", "overlay", "opengl", "openglhq", "openglnb", "surface"});
    output.setVisibleItemCount(10);
    output.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label frameskipLabel = new Label(composite_2, SWT.NONE);
    frameskipLabel.setText("Frameskip");

    frameskip = new Combo(composite_2, SWT.READ_ONLY);
    frameskip.setItems(new String[] {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"});
    frameskip.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label scalerLabel = new Label(composite_2, SWT.NONE);
    scalerLabel.setText("Scaler");

    scaler = new Combo(composite_2, SWT.READ_ONLY);
    scaler.setItems(new String[] {"none", "normal2x", "normal3x", "advmame2x", "advmame3x", "advinterp2x", "advinterp3x", "tv2x", "tv3x",
            "rgb2x", "rgb3x", "scan2x", "scan3x", "supereagle", "super2xsai", "2xsai", "hq2x", "hq3x", "hardware2x", "hardware3x"});
    scaler.setVisibleItemCount(15);
    scaler.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label fullscreenLabel = new Label(composite_2, SWT.NONE);
    fullscreenLabel.setText("Fullscreen");

    fullscreen = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label doubleBufferingLabel = new Label(composite_2, SWT.NONE);
    doubleBufferingLabel.setText("Double Buffering");

    fulldouble = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final Label resolutionLabel = new Label(composite_2, SWT.NONE);
    resolutionLabel.setText("Fullscreen Resolution");

    fullresolution = new Combo(composite_2, SWT.READ_ONLY);
    fullresolution.setItems(new String[] {"original", "320x200", "640x480", "800x600", "1024x768", "1280x768", "1280x960", "1280x1024"});
    fullresolution.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label windowResolutionLabel = new Label(composite_2, SWT.NONE);
    windowResolutionLabel.setText("Window Resolution");

    windowresolution = new Combo(composite_2, SWT.READ_ONLY);
    windowresolution.setItems(new String[] {"original", "320x200", "640x480", "800x600", "1024x768", "1280x768", "1280x960", "1280x1024"});
    windowresolution.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label aspectCorrectionLabel = new Label(composite_2, SWT.NONE);
    aspectCorrectionLabel.setText("Aspect Correction");

    aspect = new Button(composite_2, SWT.CHECK);
    new Label(composite_2, SWT.NONE);
    new Label(composite_2, SWT.NONE);

    final TabItem machineTabItem = new TabItem(tabFolder, SWT.NONE);
    machineTabItem.setText("Machine");

    final Composite composite_3 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_2 = new GridLayout();
    composite_3.setLayout(gridLayout_2);
    machineTabItem.setControl(composite_3);

    final Group cpuGroup = new Group(composite_3, SWT.NONE);
    cpuGroup.setText("CPU");
    final GridLayout gridLayout_3 = new GridLayout();
    gridLayout_3.numColumns = 6;
    cpuGroup.setLayout(gridLayout_3);

    final Label machineLabel = new Label(cpuGroup, SWT.NONE);
    machineLabel.setText("Machine");

    machine = new Combo(cpuGroup, SWT.READ_ONLY);
    machine.setItems(new String[] {"cga", "hercules", "pcjr", "tandy", "vga"});
    machine.setLayoutData(new GridData());
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Label coreLabel = new Label(cpuGroup, SWT.NONE);
    coreLabel.setText("Core");

    core = new Combo(cpuGroup, SWT.READ_ONLY);
    core.setItems(new String[] {"dynamic", "full", "normal", "simple", "auto"});
    core.setLayoutData(new GridData());
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Label cyclesLabel = new Label(cpuGroup, SWT.NONE);
    cyclesLabel.setText("Cycles");

    cycles = new Combo(cpuGroup, SWT.NONE);
    cycles.setItems(new String[] {"500", "750", "1000", "2000", "3000", "4000", "5000", "7500", "10000", "12500", "15000", "17500", "20000", "25000", "30000", "32500", "35000", "40000", "45000", "50000", "55000", "60000", "auto", "max"});
    cycles.setLayoutData(new GridData());
    cycles.setVisibleItemCount(15);

    final Label upLabel = new Label(cpuGroup, SWT.NONE);
    upLabel.setLayoutData(new GridData());
    upLabel.setText("Up");

    cycles_up = new Combo(cpuGroup, SWT.NONE);
    cycles_up.setItems(new String[] {"20", "50", "100", "500", "1000", "2000", "5000", "10000"});
    cycles_up.setLayoutData(new GridData());

    final Label downLabel = new Label(cpuGroup, SWT.NONE);
    downLabel.setLayoutData(new GridData());
    downLabel.setText("Down");

    cycles_down = new Combo(cpuGroup, SWT.NONE);
    cycles_down.setItems(new String[] {"20", "50", "100", "500", "1000", "2000", "5000", "10000"});
    cycles_down.setLayoutData(new GridData());

    final Label timesynchedLabel = new Label(cpuGroup, SWT.NONE);
    timesynchedLabel.setText("Timesynched");

    timesynched = new Button(cpuGroup, SWT.CHECK);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);
    new Label(cpuGroup, SWT.NONE);

    final Group memoryGroup = new Group(composite_3, SWT.NONE);
    memoryGroup.setText("Memory");
    memoryGroup.setLayoutData(new GridData());
    final GridLayout gridLayout_4 = new GridLayout();
    gridLayout_4.numColumns = 3;
    memoryGroup.setLayout(gridLayout_4);

    final Label sizeLabel = new Label(memoryGroup, SWT.NONE);
    sizeLabel.setText("Size");

    memsize = new Combo(memoryGroup, SWT.READ_ONLY);
    memsize.setItems(new String[] {"0", "1", "2", "4", "8", "16", "32", "63"});
    memsize.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(memoryGroup, SWT.NONE);

    final Label xmsLabel = new Label(memoryGroup, SWT.NONE);
    xmsLabel.setText("XMS");

    xms = new Button(memoryGroup, SWT.CHECK);
    new Label(memoryGroup, SWT.NONE);

    final Label emsLabel = new Label(memoryGroup, SWT.NONE);
    emsLabel.setText("EMS");

    ems = new Button(memoryGroup, SWT.CHECK);
    new Label(memoryGroup, SWT.NONE);

    final Label umbLabel = new Label(memoryGroup, SWT.NONE);
    umbLabel.setText("UMB");

    umb = new Combo(memoryGroup, SWT.READ_ONLY);
    umb.setItems(new String[] {"false", "true", "max"});
    umb.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(memoryGroup, SWT.NONE);

    final Label loadfixLabel = new Label(memoryGroup, SWT.NONE);
    loadfixLabel.setText("Loadfix");

    loadfix = new Button(memoryGroup, SWT.CHECK);

    loadfix_value = new Combo(memoryGroup, SWT.NONE);
    loadfix_value.setItems(new String[] {"1", "63", "127"});
    loadfix_value.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final TabItem audioTabItem = new TabItem(tabFolder, SWT.NONE);
    audioTabItem.setText("Audio");

    final Composite composite_4 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_9 = new GridLayout();
    gridLayout_9.numColumns = 3;
    composite_4.setLayout(gridLayout_9);
    audioTabItem.setControl(composite_4);

    final Group generalGroup = new Group(composite_4, SWT.NONE);
    generalGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    final GridLayout gridLayout_6 = new GridLayout();
    gridLayout_6.numColumns = 2;
    generalGroup.setLayout(gridLayout_6);
    generalGroup.setText("General");

    final Label silentModeLabel = new Label(generalGroup, SWT.NONE);
    silentModeLabel.setText("Silent Mode");

    nosound = new Button(generalGroup, SWT.CHECK);

    final Label sampleRateLabel = new Label(generalGroup, SWT.NONE);
    sampleRateLabel.setText("Sample Rate");

    rate = new Combo(generalGroup, SWT.READ_ONLY);
    rate.setItems(new String[] {"11025", "22050", "44100"});
    rate.setLayoutData(new GridData());

    final Label blockSizeLabel = new Label(generalGroup, SWT.NONE);
    blockSizeLabel.setText("Block Size");

    blocksize = new Combo(generalGroup, SWT.READ_ONLY);
    blocksize.setItems(new String[] {"512", "1024", "2048", "3072", "4096", "8192"});
    blocksize.setLayoutData(new GridData());

    final Label prebufferLabel = new Label(generalGroup, SWT.NONE);
    prebufferLabel.setText("Prebuffer");

    prebuffer = new Combo(generalGroup, SWT.NONE);
    prebuffer.setItems(new String[] {"10"});
    prebuffer.setLayoutData(new GridData());

    final Label mpu401Label = new Label(generalGroup, SWT.NONE);
    mpu401Label.setText("MPU-401");

    mpu401 = new Combo(generalGroup, SWT.READ_ONLY);
    mpu401.setItems(new String[] {"none", "intelligent", "uart"});
    mpu401.setLayoutData(new GridData());

    final Label midiDeviceLabel = new Label(generalGroup, SWT.NONE);
    midiDeviceLabel.setText("Midi Device");

    device = new Combo(generalGroup, SWT.READ_ONLY);
    device.setItems(new String[] {"alsa", "default", "coreaudio", "none", "oss", "win32"});
    device.setLayoutData(new GridData());

    final Label configLabel = new Label(generalGroup, SWT.NONE);
    configLabel.setText("Config");

    midi_config = new Text(generalGroup, SWT.BORDER);
    midi_config.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group soundblasterGroup = new Group(composite_4, SWT.NONE);
    soundblasterGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    soundblasterGroup.setText("Soundblaster");
    final GridLayout gridLayout_7 = new GridLayout();
    gridLayout_7.numColumns = 2;
    soundblasterGroup.setLayout(gridLayout_7);

    final Label typeLabel = new Label(soundblasterGroup, SWT.NONE);
    typeLabel.setText("Type");

    sbtype = new Combo(soundblasterGroup, SWT.READ_ONLY);
    sbtype.setItems(new String[] {"none", "sb1", "sb2", "sbpro1", "sbpro2", "sb16"});
    sbtype.setLayoutData(new GridData());

    final Label oplRateLabel = new Label(soundblasterGroup, SWT.NONE);
    oplRateLabel.setText("OPL Rate");

    oplrate = new Combo(soundblasterGroup, SWT.READ_ONLY);
    oplrate.setItems(new String[] {"11025", "22050", "44100"});
    oplrate.setLayoutData(new GridData());

    final Label oplModeLabel = new Label(soundblasterGroup, SWT.NONE);
    oplModeLabel.setLayoutData(new GridData());
    oplModeLabel.setText("OPL Mode");

    oplmode = new Combo(soundblasterGroup, SWT.READ_ONLY);
    oplmode.setItems(new String[] {"auto", "cms", "opl2", "dualopl2", "opl3"});
    oplmode.setLayoutData(new GridData());

    final Label addressLabel = new Label(soundblasterGroup, SWT.NONE);
    addressLabel.setText("Address");

    sbbase = new Combo(soundblasterGroup, SWT.READ_ONLY);
    sbbase.setItems(new String[] {"210", "220", "240", "260", "280"});
    sbbase.setLayoutData(new GridData());

    final Label irqLabel = new Label(soundblasterGroup, SWT.NONE);
    irqLabel.setText("IRQ");

    irq = new Combo(soundblasterGroup, SWT.READ_ONLY);
    irq.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq.setLayoutData(new GridData());

    final Label dmaLabel = new Label(soundblasterGroup, SWT.NONE);
    dmaLabel.setText("DMA");

    dma = new Combo(soundblasterGroup, SWT.READ_ONLY);
    dma.setItems(new String[] {"0", "1", "3"});
    dma.setLayoutData(new GridData());

    final Label hdmaLabel = new Label(soundblasterGroup, SWT.NONE);
    hdmaLabel.setLayoutData(new GridData());
    hdmaLabel.setText("HDMA");

    hdma = new Combo(soundblasterGroup, SWT.READ_ONLY);
    hdma.setItems(new String[] {"5", "6", "7"});
    hdma.setLayoutData(new GridData());

    final Label mixerLabel = new Label(soundblasterGroup, SWT.NONE);
    mixerLabel.setText("Mixer");

    mixer = new Button(soundblasterGroup, SWT.CHECK);

    final Group gusGroup = new Group(composite_4, SWT.NONE);
    gusGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    gusGroup.setText("Gravis Ultrasound");
    final GridLayout gridLayout_8 = new GridLayout();
    gridLayout_8.numColumns = 2;
    gusGroup.setLayout(gridLayout_8);

    final Label enableLabel_1 = new Label(gusGroup, SWT.NONE);
    enableLabel_1.setLayoutData(new GridData());
    enableLabel_1.setText("Enable");

    gus = new Button(gusGroup, SWT.CHECK);
    gus.setLayoutData(new GridData());

    final Label sampleRateLabel_2 = new Label(gusGroup, SWT.NONE);
    sampleRateLabel_2.setText("Sample Rate");

    gusrate = new Combo(gusGroup, SWT.READ_ONLY);
    gusrate.setItems(new String[] {"11025", "22050", "44100"});
    gusrate.setLayoutData(new GridData());

    final Label addressLabel_1 = new Label(gusGroup, SWT.NONE);
    addressLabel_1.setText("Address");

    gusbase = new Combo(gusGroup, SWT.READ_ONLY);
    gusbase.setItems(new String[] {"210", "220", "240", "260", "280"});
    gusbase.setLayoutData(new GridData());

    final Label irqLabel_1 = new Label(gusGroup, SWT.NONE);
    irqLabel_1.setText("IRQ1");

    irq1 = new Combo(gusGroup, SWT.READ_ONLY);
    irq1.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq1.setLayoutData(new GridData());

    final Label irq2Label = new Label(gusGroup, SWT.NONE);
    irq2Label.setText("IRQ2");

    irq2 = new Combo(gusGroup, SWT.READ_ONLY);
    irq2.setItems(new String[] {"3", "5", "7", "10", "11"});
    irq2.setLayoutData(new GridData());

    final Label dmaLabel_1 = new Label(gusGroup, SWT.NONE);
    dmaLabel_1.setText("DMA1");

    dma1 = new Combo(gusGroup, SWT.READ_ONLY);
    dma1.setItems(new String[] {"0", "1", "3"});
    dma1.setLayoutData(new GridData());

    final Label hdmaLabel_1 = new Label(gusGroup, SWT.NONE);
    hdmaLabel_1.setLayoutData(new GridData());
    hdmaLabel_1.setText("DMA2");

    dma2 = new Combo(gusGroup, SWT.READ_ONLY);
    dma2.setItems(new String[] {"0", "1", "3"});
    dma2.setLayoutData(new GridData());

    final Label ultradirLabel = new Label(gusGroup, SWT.NONE);
    ultradirLabel.setText("Ultradir");

    ultradir = new Text(gusGroup, SWT.BORDER);
    ultradir.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group speakerGroup = new Group(composite_4, SWT.NONE);
    speakerGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    speakerGroup.setText("PC Speaker");
    final GridLayout gridLayout_10 = new GridLayout();
    gridLayout_10.numColumns = 2;
    speakerGroup.setLayout(gridLayout_10);

    final Label enableLabel = new Label(speakerGroup, SWT.NONE);
    enableLabel.setText("Enable");

    pcspeaker = new Button(speakerGroup, SWT.CHECK);

    final Label sampleRateLabel_1 = new Label(speakerGroup, SWT.NONE);
    sampleRateLabel_1.setText("Sample Rate");

    pcrate = new Combo(speakerGroup, SWT.READ_ONLY);
    pcrate.setItems(new String[] {"11025", "22050", "44100"});
    pcrate.setLayoutData(new GridData());

    final Group tandyGroup = new Group(composite_4, SWT.NONE);
    tandyGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    tandyGroup.setText("Tandy");
    final GridLayout gridLayout_11 = new GridLayout();
    gridLayout_11.numColumns = 2;
    tandyGroup.setLayout(gridLayout_11);

    final Label enableLabel_2 = new Label(tandyGroup, SWT.NONE);
    enableLabel_2.setText("Enable");

    tandy = new Combo(tandyGroup, SWT.READ_ONLY);
    tandy.setItems(new String[] {"auto", "off", "on"});
    tandy.setLayoutData(new GridData());

    final Label sampleRateLabel_1_1 = new Label(tandyGroup, SWT.NONE);
    sampleRateLabel_1_1.setText("Sample Rate");

    tandyrate = new Combo(tandyGroup, SWT.READ_ONLY);
    tandyrate.setItems(new String[] {"11025", "22050", "44100"});
    tandyrate.setLayoutData(new GridData());

    final Group disneyGroup = new Group(composite_4, SWT.NONE);
    disneyGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    disneyGroup.setText("Disney Sound Source");
    final GridLayout gridLayout_12 = new GridLayout();
    gridLayout_12.numColumns = 2;
    disneyGroup.setLayout(gridLayout_12);

    final Label enableLabel_3 = new Label(disneyGroup, SWT.NONE);
    enableLabel_3.setText("Enable");

    disney = new Button(disneyGroup, SWT.CHECK);

    final TabItem inputTabItem = new TabItem(tabFolder, SWT.NONE);
    inputTabItem.setText("I/O");

    final Composite composite_5 = new Composite(tabFolder, SWT.NONE);
    final GridLayout gridLayout_17 = new GridLayout();
    gridLayout_17.numColumns = 3;
    composite_5.setLayout(gridLayout_17);
    inputTabItem.setControl(composite_5);

    final Group mouseGroup = new Group(composite_5, SWT.NONE);
    mouseGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    mouseGroup.setText("Mouse");
    final GridLayout gridLayout_13 = new GridLayout();
    gridLayout_13.numColumns = 2;
    mouseGroup.setLayout(gridLayout_13);

    final Label autolockLabel = new Label(mouseGroup, SWT.NONE);
    autolockLabel.setText("Autolock");

    autolock = new Button(mouseGroup, SWT.CHECK);

    final Label sensitivityLabel = new Label(mouseGroup, SWT.NONE);
    sensitivityLabel.setText("Sensitivity");

    sensitivity = new Combo(mouseGroup, SWT.READ_ONLY);
    sensitivity.setItems(new String[] {"10", "20", "30", "40", "50", "60", "70", "80", "90", "100", "125", "150", "175", "200", "250", "300", "350", "400", "450", "500", "550", "600", "700", "800", "900", "1000"});
    sensitivity.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group keyboardGroup = new Group(composite_5, SWT.NONE);
    keyboardGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
    keyboardGroup.setText("Keyboard");
    final GridLayout gridLayout_14 = new GridLayout();
    gridLayout_14.numColumns = 2;
    keyboardGroup.setLayout(gridLayout_14);

    final Label useScancodesLabel = new Label(keyboardGroup, SWT.NONE);
    useScancodesLabel.setText("Use Scancodes");

    usescancodes = new Button(keyboardGroup, SWT.CHECK);

    final Label mapperFileLabel = new Label(keyboardGroup, SWT.NONE);
    mapperFileLabel.setText("Mapper File");

    mapperfile = new Text(keyboardGroup, SWT.BORDER);
    mapperfile.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group joystickGroup = new Group(composite_5, SWT.NONE);
    joystickGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    joystickGroup.setText("Joystick");
    final GridLayout gridLayout_15 = new GridLayout();
    gridLayout_15.numColumns = 2;
    joystickGroup.setLayout(gridLayout_15);

    final Label typeLabel_1 = new Label(joystickGroup, SWT.NONE);
    typeLabel_1.setText("Type");

    joysticktype = new Combo(joystickGroup, SWT.READ_ONLY);
    joysticktype.setItems(new String[] {"none", "2axis", "4axis", "ch", "fcs"});
    joysticktype.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group modemGroup = new Group(composite_5, SWT.NONE);
    modemGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1));
    modemGroup.setText("Modem");
    final GridLayout gridLayout_16 = new GridLayout();
    gridLayout_16.numColumns = 2;
    modemGroup.setLayout(gridLayout_16);

    final Label serial1Label = new Label(modemGroup, SWT.NONE);
    serial1Label.setText("Serial 1");

    serial1 = new Text(modemGroup, SWT.BORDER);
    serial1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial2Label = new Label(modemGroup, SWT.NONE);
    serial2Label.setText("Serial 2");

    serial2 = new Text(modemGroup, SWT.BORDER);
    serial2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial3Label = new Label(modemGroup, SWT.NONE);
    serial3Label.setText("Serial 3");

    serial3 = new Text(modemGroup, SWT.BORDER);
    serial3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label serial4Label = new Label(modemGroup, SWT.NONE);
    serial4Label.setText("Serial 4");

    serial4 = new Text(modemGroup, SWT.BORDER);
    serial4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Group networkGroup = new Group(composite_5, SWT.NONE);
    networkGroup.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, false));
    networkGroup.setText("Network");
    final GridLayout gridLayout_20 = new GridLayout();
    gridLayout_20.numColumns = 2;
    networkGroup.setLayout(gridLayout_20);

    final Label ipxLabel = new Label(networkGroup, SWT.NONE);
    ipxLabel.setText("Enable IPX");

    ipx = new Button(networkGroup, SWT.CHECK);
    new Label(composite_5, SWT.NONE);

    mountingTabItem = new TabItem(tabFolder, SWT.NONE);
    mountingTabItem.setText("Mounting");

    final Composite composite_6 = new Composite(tabFolder, SWT.NONE);
    composite_6.setLayout(new GridLayout());
    mountingTabItem.setControl(composite_6);

    final Group mountGroup = new Group(composite_6, SWT.NONE);
    mountGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    mountGroup.setText("Mounting Overview");
    final GridLayout gridLayout_19 = new GridLayout();
    gridLayout_19.numColumns = 2;
    mountGroup.setLayout(gridLayout_19);

    mountingpoints = new List(mountGroup, SWT.V_SCROLL | SWT.BORDER);
    final GridData gridData_2 = new GridData(SWT.FILL, SWT.FILL, true, false, 1, 3);
    mountingpoints.setLayoutData(gridData_2);

    final Button addButton = new Button(mountGroup, SWT.NONE);
    addButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
    addButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        final EditMountDialog addMountDialog = new EditMountDialog(shell);
        java.util.List<Character> freeDriveletters = Mount.getFreeDriveletters(false, mountingpoints.getItems());
        if (freeDriveletters.size() > 0)
        	addMountDialog.setDefaultDriveletter(freeDriveletters.get(0));
        String mount = (String)addMountDialog.open();
        if (mount != null) {
          mountingpoints.add(mount);
          mountingpoints.select(mountingpoints.getItemCount() - 1);
        }
      }
    });
    addButton.setText("Add");

    final Button editButton = new Button(mountGroup, SWT.NONE);
    editButton.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
    editButton.setText("Edit");
    editButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (mountingpoints.getSelectionIndex() != -1) {
          final EditMountDialog editMountDialog = new EditMountDialog(shell);
          editMountDialog.setMount(mountingpoints.getItem(mountingpoints.getSelectionIndex()));
          String mount = (String)editMountDialog.open();
          if (mount != null) {
            mountingpoints.setItem(mountingpoints.getSelectionIndex(), mount);
          }
        }
      }
    });

    final Button removeButton = new Button(mountGroup, SWT.NONE);
    removeButton.setText("Remove");
    removeButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (mountingpoints.getSelectionIndex() != -1) {
          mountingpoints.remove(mountingpoints.getSelectionIndex());
        } else if (mountingpoints.getItemCount() == 1) {
        	mountingpoints.removeAll();
        }
      }
    });

    final Group executeGroup = new Group(composite_6, SWT.NONE);
    executeGroup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    executeGroup.setText("Execute");
    final GridLayout gridLayout_18 = new GridLayout();
    gridLayout_18.numColumns = 5;
    executeGroup.setLayout(gridLayout_18);

    dosButton = new Button(executeGroup, SWT.RADIO);
    dosButton.setText("Dos");

    final Label mainExeLabel = new Label(executeGroup, SWT.NONE);
    mainExeLabel.setText("Main");

    main = new Text(executeGroup, SWT.BORDER);
    final GridData gridData_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1);
    main.setLayoutData(gridData_1);

    final Button mainBrowseButton = new Button(executeGroup, SWT.NONE);
    mainBrowseButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        FileDialog dialog = new FileDialog (shell, SWT.OPEN);
        dialog.setFilterNames (new String [] {"Executables"});
        dialog.setFilterExtensions (new String [] {"*.com;*.COM;*.exe;*.EXE;*.bat;*.BAT"});
        String result = dialog.open();
        if (result != null) {
          main.setText(result);
          addMountIfNoMounts(result);
        }
      }
    });
    mainBrowseButton.setText("Browse..");
    new Label(executeGroup, SWT.NONE);
    new Label(executeGroup, SWT.NONE);

    final Label parametersLabel = new Label(executeGroup, SWT.NONE);
    parametersLabel.setLayoutData(new GridData());
    parametersLabel.setText("Parameters");

    main_params = new Text(executeGroup, SWT.BORDER);
    main_params.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(executeGroup, SWT.NONE);
    new Label(executeGroup, SWT.NONE);

    final Label setupLabel = new Label(executeGroup, SWT.NONE);
    setupLabel.setLayoutData(new GridData());
    setupLabel.setText("Setup");

    setup = new Text(executeGroup, SWT.BORDER);
    setup.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

    final Button setupBrowseButton = new Button(executeGroup, SWT.NONE);
    setupBrowseButton.setText("Browse..");
    setupBrowseButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        FileDialog dialog = new FileDialog (shell, SWT.OPEN);
        dialog.setFilterNames (new String [] {"Executables"});
        dialog.setFilterExtensions (new String [] {"*.com;*.COM;*.exe;*.EXE;*.bat;*.BAT"});
        String result = dialog.open();
        if (result != null) {
          setup.setText(result);
          addMountIfNoMounts(result);
        }
      }
    });
    new Label(executeGroup, SWT.NONE);
    new Label(executeGroup, SWT.NONE);

    final Label parametersLabel_1 = new Label(executeGroup, SWT.NONE);
    parametersLabel_1.setLayoutData(new GridData());
    parametersLabel_1.setText("Parameters");

    setup_params = new Text(executeGroup, SWT.BORDER);
    setup_params.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
    new Label(executeGroup, SWT.NONE);

    final Label label_1 = new Label(executeGroup, SWT.SEPARATOR | SWT.HORIZONTAL);
    label_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 5, 1));

    booterButton = new Button(executeGroup, SWT.RADIO);
    booterButton.setText("Booter");

    final Label image1Label = new Label(executeGroup, SWT.NONE);
    image1Label.setText("Image 1");

    img1 = new Text(executeGroup, SWT.BORDER);
    img1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

    final Button img1BrowseButton = new Button(executeGroup, SWT.NONE);
    img1BrowseButton.setLayoutData(new GridData());
    img1BrowseButton.setText("Browse..");
    img1BrowseButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        FileDialog dialog = new FileDialog (shell, SWT.OPEN);
        dialog.setFilterNames (new String [] {"Booter Disk Images", "*.*"});
        dialog.setFilterExtensions (
        		new String [] {"*.cp2;*.CP2;*.dcf;*.DCF;*.img;*.IMG;*.jrc;*.JRC;*.td0;*.TD0", "*.*"});
        String result = dialog.open();
        if (result != null) {
          img1.setText(result);
          addMountIfNoMounts(result);
        }
      }
    });
    
    new Label(executeGroup, SWT.NONE);

    final Label image2Label = new Label(executeGroup, SWT.NONE);
    image2Label.setText("Image 2");

    img2 = new Text(executeGroup, SWT.BORDER);
    img2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

    final Button img2BrowseButton = new Button(executeGroup, SWT.NONE);
    img2BrowseButton.setText("Browse..");
    img2BrowseButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        FileDialog dialog = new FileDialog (shell, SWT.OPEN);
        dialog.setFilterNames (new String [] {"Booter Disk Images", "*.*"});
        dialog.setFilterExtensions (
        		new String [] {"*.cp2;*.CP2;*.dcf;*.DCF;*.img;*.IMG;*.jrc;*.JRC;*.td0;*.TD0", "*.*"});
        String result = dialog.open();
        if (result != null) {
          img2.setText(result);
          addMountIfNoMounts(result);
        }
      }
    });

    final Composite composite_7 = new Composite(shell, SWT.NONE);
    composite_7.setLayout(new RowLayout());
    composite_7.setLayoutData(BorderLayout.SOUTH);

    final Button okButton = new Button(composite_7, SWT.NONE);
    shell.setDefaultButton(okButton);
    final RowData rowData = new RowData();
    rowData.width = 80;
    okButton.setLayoutData(rowData);
    okButton.setText("OK");
    
    final Button cancelButton = new Button(composite_7, SWT.NONE);
    final RowData rowData_1 = new RowData();
    rowData_1.width = 80;
    cancelButton.setLayoutData(rowData_1);
    cancelButton.setText("Cancel");
    cancelButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        result = null;
        shell.close();
      }
    });
    
    final Label capturesLabel = new Label(composite_1, SWT.NONE);
    capturesLabel.setLayoutData(new GridData());
    capturesLabel.setText("Captures");

    captures = new Text(composite_1, SWT.BORDER);
    final GridData gridData_3 = new GridData(SWT.FILL, SWT.CENTER, false, false, 2, 1);
    captures.setLayoutData(gridData_3);
    captures.setEditable(false);

    final Label link1Label = new Label(composite, SWT.NONE);
    link1Label.setText("Link 1");

    link1 = new Text(composite, SWT.BORDER);
    link1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label link2Label = new Label(composite, SWT.NONE);
    link2Label.setLayoutData(new GridData());
    link2Label.setText("Link 2");

    link2 = new Text(composite, SWT.BORDER);
    link2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label statusLabel = new Label(composite, SWT.NONE);
    statusLabel.setText("Status");

    status = new Combo(composite, SWT.NONE);
    status.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

    final Label favoriteLabel = new Label(composite, SWT.NONE);
    favoriteLabel.setText("Favorite");

    final Button favorite = new Button(composite, SWT.CHECK);

    final Label notesLabel = new Label(composite, SWT.NONE);
    notesLabel.setText("Notes");

    notes = new Text(composite, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER | SWT.WRAP);
    final GridData gridData = new GridData(SWT.FILL, SWT.FILL, true, true, 3, 1);
    notes.setLayoutData(gridData);
    
    // init values
    for (Developer dev: developersList) developer.add(dev.getName());
    for (Publisher publ: publishersList) publisher.add(publ.getName());
    for (Genre gen: genresList) genre.add(gen.getName());
    for (Year yr: yearsList) year.add(yr.getYear());
    for (Status stat: statusList) status.add(stat.getStat());
    for (DosboxVersion dbv: dbversionsList) dbversion.add(dbv.getTitle());
    for (Template templ: templatesList) template.add(templ.getTitle());

    if (result != null) {
      // meaning we are essentially editing an existing profile
      // so we need to set previous values
      title.setText(((Profile)result).getTitle());
      developer.setText(((Profile)result).getDeveloperName());
      publisher.setText(((Profile)result).getPublisherName());
      genre.setText(((Profile)result).getGenre());
      year.setText(((Profile)result).getYear());
      link1.setText(((Profile)result).getLink1());
      link2.setText(((Profile)result).getLink2());
      status.setText(((Profile)result).getStatus());
      favorite.setSelection(((Profile)result).isFavorite());
      notes.setText(((Profile)result).getNotes());
      int tableIndexVersionIndex = ListUtils.findTIDosboxVersion(dbversionsList, ((Profile)result).getDbversionId());
      dbversion.select(tableIndexVersionIndex);
      setup.setText(((Profile)result).getSetup());
      setup_params.setText(((Profile)result).getSetupParameters());
      
      DosboxVersion selectedDBVersion = dbversionsList.get(tableIndexVersionIndex);
      profileConf = new Configuration(selectedDBVersion.getPath() + MainWindow.DOSBOX_CONF);
      profileConf.parseConfigurationFile(MainWindow.PROFILES_DIR + System.getProperty("file.separator") + ((Profile)result).getId() + ".conf");
    } else {
      // set default values for new profile
      int defaultDosboxVersionIndex = ListUtils.findTIDefaultDosboxVersion(dbversionsList);
      dbversion.select(defaultDosboxVersionIndex);
      DosboxVersion defaultDBVersion = dbversionsList.get(defaultDosboxVersionIndex);
      Configuration defaultConf = new Configuration(defaultDBVersion.getPath() + MainWindow.DOSBOX_CONF);
      int defaultTemplateIndex = ListUtils.findTIDefaultTemplate(templatesList);
      if (defaultTemplateIndex != -1) {
        Template defaultTemplate = templatesList.get(defaultTemplateIndex);
        template.select(defaultTemplateIndex);
        defaultConf.parseConfigurationFile(MainWindow.TEMPLATES_DIR + System.getProperty("file.separator") + defaultTemplate.getId() + ".conf");
      } else {
      	// enabled 'exit afterwards' hardcoded, by default
      	defaultConf.setValue(Configuration.AUTOEXEC, Configuration.AUTOEXEC, "exit");
      }
      
      profileConf = new Configuration(defaultConf);
    }
    
    main.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(true);
        booterButton.setSelection(false);
      }
    });
    main_params.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(true);
        booterButton.setSelection(false);
      }
    });
    setup.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(true);
        booterButton.setSelection(false);
      }
    });
    setup_params.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(true);
        booterButton.setSelection(false);
      }
    });
    img1.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(false);
        booterButton.setSelection(true);
      }
    });
    img2.addModifyListener(new ModifyListener() {
      public void modifyText(final ModifyEvent e) {
        dosButton.setSelection(false);
        booterButton.setSelection(true);
      }
    });
    okButton.addSelectionListener(new SelectionAdapter() {
      public void widgetSelected(final SelectionEvent e) {
        if (!isValid())
          return;
        int profileId = -1;
        int devIndex = -1;
        int publIndex = -1;
        int genIndex = -1;
        int yrIndex = -1;
        int statIndex = -1;
        if (developer.getSelectionIndex() == -1)
          developer.select(developer.indexOf(developer.getText()));
        if (developer.getSelectionIndex() != -1)
          devIndex = developersList.get(developer.getSelectionIndex()).getId();
        if (publisher.getSelectionIndex() == -1)
          publisher.select(publisher.indexOf(publisher.getText()));
        if (publisher.getSelectionIndex() != -1)
          publIndex = publishersList.get(publisher.getSelectionIndex()).getId();
        if (genre.getSelectionIndex() == -1)
          genre.select(genre.indexOf(genre.getText()));
        if (genre.getSelectionIndex() != -1)
          genIndex = genresList.get(genre.getSelectionIndex()).getId();
        if (year.getSelectionIndex() == -1)
          year.select(year.indexOf(year.getText()));
        if (year.getSelectionIndex() != -1)
          yrIndex = yearsList.get(year.getSelectionIndex()).getId();
        if (status.getSelectionIndex() == -1)
          status.select(status.indexOf(status.getText()));
        if (status.getSelectionIndex() != -1)
          statIndex = statusList.get(status.getSelectionIndex()).getId();
        int dbversionId = dbversionsList.get(dbversion.getSelectionIndex()).getId();
        profileId = db.addOrEditProfile(title.getText(), developer.getText(), publisher.getText(),
        				genre.getText(), year.getText(), status.getText(), setup.getText(), setup_params.getText(),
                notes.getText(), link1.getText(), link2.getText(), favorite.getSelection(),
                devIndex, publIndex, genIndex, yrIndex, statIndex, dbversionId,
                result != null ? ((Profile)result).getId(): -1);
        result = new Profile(profileId, title.getText(), developer.getText(), publisher.getText(),
        				genre.getText(), year.getText(), dbversionId, setup.getText(), setup_params.getText(),
                notes.getText(), link1.getText(), link2.getText(), favorite.getSelection(), status.getText());
        DosboxVersion selectedDBVersion = dbversionsList.get(dbversion.getSelectionIndex());
        Configuration dosboxConf = new Configuration(selectedDBVersion.getPath() + MainWindow.DOSBOX_CONF);
        getConfigurationBySettings(profileConf, false);
        MainWindow.setCapturesDir(profileConf, profileId, null);
        if (selectedDBVersion.isMultiConfig())
          profileConf.substract(dosboxConf);
        profileConf.saveToFile(MainWindow.PROFILES_DIR + System.getProperty("file.separator") + profileId + ".conf");
        
        shell.close();
      }
    });

    selectSettingsByConfiguration(profileConf);
  }
  
  private String getAutoexecSectionBySettings(boolean isSetup) {
    if (dosButton.getSelection()) {
      return new Autoexec(mountingpoints.getItems(), loadfix.getSelection(), loadfix_value.getText(),
              main.getText(), setup.getText(), main_params.getText(), setup_params.getText(),
              "", "", exit.getSelection()).toString(isSetup);
    } else {
      return new Autoexec(mountingpoints.getItems(), loadfix.getSelection(), loadfix_value.getText(),
              "", "", "", "", img1.getText(), img2.getText(), exit.getSelection()).toString(isSetup);
    }
  }
  
  private void selectSettingsByAutoexec(Configuration profileConf) {
    Autoexec autoexec = new Autoexec(profileConf.getValue(Configuration.AUTOEXEC, Configuration.AUTOEXEC));
    exit.setSelection(autoexec.isExit());
    loadfix.setSelection(autoexec.isLoadfix());
    loadfix_value.setText(autoexec.getLoadfixValue());
    if (mountingpoints.getItemCount() == 0) {
      mountingpoints.setItems(autoexec.getMountingpoints());
      mountingpoints.setSelection(0);
    }
    if (main.getText().equals("") && !autoexec.getMain().equals("")) {
      main.setText(autoexec.getMain());
      main_params.setText(autoexec.getMainParameters());
    }
    if (img1.getText().equals("") && !autoexec.getImg1().equals("")) {
      img1.setText(autoexec.getImg1());
      img2.setText(autoexec.getImg2());
    }
    dosButton.setSelection(!autoexec.isBooter());
    booterButton.setSelection(autoexec.isBooter());
  }
  
  private void selectSettingsByConfiguration(Configuration profileConf) {
    String priority = profileConf.getValue("sdl", "priority");
    String[] priorities = priority.split(",");
    if ((priorities != null) && (priorities.length == 2)) {
      priority_active.setText(priorities[0]);
      priority_inactive.setText(priorities[1]);
    }
    waitonerror.setSelection(Boolean.valueOf(profileConf.getValue("sdl", "waitonerror")).booleanValue());
    language.setText(profileConf.getValue("dosbox", "language"));
    captures.setText("<automatic>");
    output.setText(profileConf.getValue("sdl", "output"));
    frameskip.setText(profileConf.getValue("render", "frameskip"));
    scaler.setText(profileConf.getValue("render", "scaler"));
    fullscreen.setSelection(Boolean.valueOf(profileConf.getValue("sdl", "fullscreen")).booleanValue());
    fulldouble.setSelection(Boolean.valueOf(profileConf.getValue("sdl", "fulldouble")).booleanValue());
    fullresolution.setText(profileConf.getValue("sdl", "fullresolution"));
    windowresolution.setText(profileConf.getValue("sdl", "windowresolution"));
    aspect.setSelection(Boolean.valueOf(profileConf.getValue("render", "aspect")).booleanValue());
    machine.setText(profileConf.getValue("dosbox", "machine"));
    core.setText(profileConf.getValue("cpu", "core"));
    cycles.setText(profileConf.getValue("cpu", "cycles"));
    cycles_up.setText(profileConf.getValue("cpu", "cycleup"));
    cycles_down.setText(profileConf.getValue("cpu", "cycledown"));
    timesynched.setSelection(Boolean.valueOf(profileConf.getValue("cpu", "timesynched")).booleanValue());
    timesynched.setEnabled(profileConf.hasValue("cpu", "timesynched"));
    memsize.setText(profileConf.getValue("dosbox", "memsize"));
    xms.setSelection(Boolean.valueOf(profileConf.getValue("dos", "xms")).booleanValue());
    ems.setSelection(Boolean.valueOf(profileConf.getValue("dos", "ems")).booleanValue());
    umb.setText(profileConf.getValue("dos", "umb"));
    nosound.setSelection(Boolean.valueOf(profileConf.getValue("mixer", "nosound")).booleanValue());
    rate.setText(profileConf.getValue("mixer", "rate"));
    blocksize.setText(profileConf.getValue("mixer", "blocksize"));
    prebuffer.setText(profileConf.getValue("mixer", "prebuffer"));
    mpu401.setText(profileConf.getValue("midi", "mpu401"));
    device.setText(profileConf.getValue("midi", "device"));
    midi_config.setText(profileConf.getValue("midi", "config"));
    sbtype.setText(profileConf.getValue("sblaster", "sbtype"));
    oplmode.setText(profileConf.getValue("sblaster", "oplmode"));
    oplrate.setText(profileConf.getValue("sblaster", "oplrate"));
    sbbase.setText(profileConf.getValue("sblaster", "sbbase"));
    irq.setText(profileConf.getValue("sblaster", "irq"));
    dma.setText(profileConf.getValue("sblaster", "dma"));
    hdma.setText(profileConf.getValue("sblaster", "hdma"));
    mixer.setSelection(Boolean.valueOf(profileConf.getValue("sblaster", "mixer")).booleanValue());
    gus.setSelection(Boolean.valueOf(profileConf.getValue("gus", "gus")).booleanValue());
    gusrate.setText(profileConf.getValue("gus", "gusrate"));
    gusbase.setText(profileConf.getValue("gus", "gusbase"));
    irq1.setText(profileConf.getValue("gus", "irq1"));
    irq2.setText(profileConf.getValue("gus", "irq2"));
    dma1.setText(profileConf.getValue("gus", "dma1"));
    dma2.setText(profileConf.getValue("gus", "dma2"));
    ultradir.setText(profileConf.getValue("gus", "ultradir"));
    pcspeaker.setSelection(Boolean.valueOf(profileConf.getValue("speaker", "pcspeaker")).booleanValue());
    pcrate.setText(profileConf.getValue("speaker", "pcrate"));
    tandy.setText(profileConf.getValue("speaker", "tandy"));
    tandyrate.setText(profileConf.getValue("speaker", "tandyrate"));
    disney.setSelection(Boolean.valueOf(profileConf.getValue("speaker", "disney")).booleanValue());
    autolock.setSelection(Boolean.valueOf(profileConf.getValue("sdl", "autolock")).booleanValue());
    sensitivity.setText(profileConf.getValue("sdl", "sensitivity"));
    usescancodes.setSelection(Boolean.valueOf(profileConf.getValue("sdl", "usescancodes")).booleanValue());
    mapperfile.setText(profileConf.getValue("sdl", "mapperfile"));
    joysticktype.setText(profileConf.getValue("bios", "joysticktype"));
    serial1.setText(profileConf.getValue("serial", "serial1"));
    serial2.setText(profileConf.getValue("serial", "serial2"));
    serial3.setText(profileConf.getValue("serial", "serial3"));
    serial4.setText(profileConf.getValue("serial", "serial4"));
    ipx.setSelection(Boolean.valueOf(profileConf.getValue("ipx", "ipx")).booleanValue());
    
    selectSettingsByAutoexec(profileConf);
  }
  
  private void getConfigurationBySettings(Configuration profileConf, boolean isSetup) {
    profileConf.setValue("sdl", "priority", priority_active.getText() + "," + priority_inactive.getText());
    profileConf.setValue("sdl", "waitonerror", String.valueOf(waitonerror.getSelection()));
    profileConf.setValue("dosbox", "language", language.getText());
    profileConf.setValue("dosbox", "captures", captures.getText());
    profileConf.setValue("sdl", "output", output.getText());
    profileConf.setValue("render", "frameskip", frameskip.getText());
    profileConf.setValue("render", "scaler", scaler.getText());
    profileConf.setValue("sdl", "fullscreen", String.valueOf(fullscreen.getSelection()));
    profileConf.setValue("sdl", "fulldouble", String.valueOf(fulldouble.getSelection()));
    profileConf.setValue("sdl", "fullresolution", fullresolution.getText());
    profileConf.setValue("sdl", "windowresolution", windowresolution.getText());
    profileConf.setValue("render", "aspect", String.valueOf(aspect.getSelection()));
    profileConf.setValue("dosbox", "machine", machine.getText());
    profileConf.setValue("cpu", "core", core.getText());
    profileConf.setValue("cpu", "cycles", cycles.getText());
    profileConf.setValue("cpu", "cycleup", cycles_up.getText());
    profileConf.setValue("cpu", "cycledown", cycles_down.getText());
    profileConf.updateValue("cpu", "timesynched", String.valueOf(timesynched.getSelection()));
    profileConf.setValue("dosbox", "memsize", memsize.getText());
    profileConf.setValue("dos", "xms", String.valueOf(xms.getSelection()));
    profileConf.setValue("dos", "ems", String.valueOf(ems.getSelection()));
    profileConf.setValue("dos", "umb", umb.getText());
    profileConf.setValue("mixer", "nosound", String.valueOf(nosound.getSelection()));
    profileConf.setValue("mixer", "rate", rate.getText());
    profileConf.setValue("mixer", "blocksize", blocksize.getText());
    profileConf.setValue("mixer", "prebuffer", prebuffer.getText());
    profileConf.setValue("midi", "mpu401", mpu401.getText());
    profileConf.setValue("midi", "device", device.getText());
    profileConf.setValue("midi", "config", midi_config.getText());
    profileConf.setValue("sblaster", "sbtype", sbtype.getText());
    profileConf.setValue("sblaster", "oplmode", oplmode.getText());
    profileConf.setValue("sblaster", "sbbase", sbbase.getText());
    profileConf.setValue("sblaster", "irq", irq.getText());
    profileConf.setValue("sblaster", "dma", dma.getText());
    profileConf.setValue("sblaster", "hdma", hdma.getText());
    profileConf.setValue("sblaster", "mixer", String.valueOf(mixer.getSelection()));
    profileConf.setValue("gus", "gus", String.valueOf(gus.getSelection()));
    profileConf.setValue("gus", "gusrate", gusrate.getText());
    profileConf.setValue("gus", "gusbase", gusbase.getText());
    profileConf.setValue("gus", "irq1", irq1.getText());
    profileConf.setValue("gus", "irq2", irq2.getText());
    profileConf.setValue("gus", "dma1", dma1.getText());
    profileConf.setValue("gus", "dma2", dma2.getText());
    profileConf.setValue("gus", "ultradir", ultradir.getText());
    profileConf.setValue("speaker", "pcspeaker", String.valueOf(pcspeaker.getSelection()));
    profileConf.setValue("speaker", "pcrate", pcrate.getText());
    profileConf.setValue("speaker", "tandy", tandy.getText());
    profileConf.setValue("speaker", "tandyrate", tandyrate.getText());
    profileConf.setValue("speaker", "disney", String.valueOf(disney.getSelection()));
    profileConf.setValue("sdl", "autolock", String.valueOf(autolock.getSelection()));
    profileConf.setValue("sdl", "sensitivity", sensitivity.getText());
    profileConf.setValue("sdl", "usescancodes", String.valueOf(usescancodes.getSelection()));
    profileConf.setValue("sdl", "mapperfile", mapperfile.getText());
    profileConf.setValue("bios", "joysticktype", joysticktype.getText());
    profileConf.setValue("serial", "serial1", serial1.getText());
    profileConf.setValue("serial", "serial2", serial2.getText());
    profileConf.setValue("serial", "serial3", serial3.getText());
    profileConf.setValue("serial", "serial4", serial4.getText());
    profileConf.setValue("ipx", "ipx", String.valueOf(ipx.getSelection()));
    
    profileConf.setValue(Configuration.AUTOEXEC, Configuration.AUTOEXEC, getAutoexecSectionBySettings(isSetup));
  }
  
  private void addMountIfNoMounts(String hostFileLocation) {
  	if (mountingpoints.getItemCount() == 0) {
    	Mount mount = new Mount(booterButton.getSelection(), hostFileLocation, mountingpoints.getItems());
    	if (!mount.getPath().equals(""))
    		mountingpoints.add(mount.toString());
    }
  }
  
  private boolean isValid() {
    Autoexec autoexec = new Autoexec(getAutoexecSectionBySettings(false));
    GeneralPurposeDialogs.initErrorDialog();
    if (title.getText().equals(""))
      GeneralPurposeDialogs.addError("You must enter a profile title", title, infoTabItem);
    if (dosButton.getSelection()) {
      if (main.getText().equals(""))
        GeneralPurposeDialogs.addError("You must enter a profile executable", main, mountingTabItem);
      else if (autoexec.getMain().equals("")) {
      	Mount mount = new Mount(false, main.getText(), mountingpoints.getItems());
      	if (!mount.getPath().equals("")) {
      		if (GeneralPurposeDialogs.confirmMessage(shell, "Would you like to add the mount location \"" + mount.getPath() + "\"?")) {
      			mountingpoints.add(mount.toString());
      			autoexec = new Autoexec(getAutoexecSectionBySettings(false));
      		}
      	}
      }
      if (autoexec.getMain().equals(""))
        GeneralPurposeDialogs.addError("You must enter a valid mount location", mountingpoints, mountingTabItem);
    } else if (booterButton.getSelection()) {
      if (img1.getText().equals(""))
        GeneralPurposeDialogs.addError("You must enter a profile booter image", img1, mountingTabItem);
      else if (autoexec.getImg1().equals("")) {
      	Mount mount = new Mount(true, img1.getText(), mountingpoints.getItems());
      	if (!mount.getPath().equals("")) {
      		if (GeneralPurposeDialogs.confirmMessage(shell, "Would you like to add the mount location \"" + mount.getPath() + "\"?")) {
      			mountingpoints.add(mount.toString());
      			autoexec = new Autoexec(getAutoexecSectionBySettings(false));
      		}
      	}
      }
      if (autoexec.getImg1().equals(""))
        GeneralPurposeDialogs.addError("You must enter a valid mount location", mountingpoints, mountingTabItem);
    }
    return !GeneralPurposeDialogs.displayErrorDialog(shell);
  }
}
